@@ -46,6 +46,7 @@ rstat::rstat()
 	TRY_nomem(m_exit_str[125] = "The command executed by SSH was killed by a signal");
 	TRY_nomem(m_exit_str[126] = "The command given to SSH cannot be run");
 	TRY_nomem(m_exit_str[127] = "The command given to SSH cannot be found");
+	TRY_nomem(m_exit_str[255] = "An SSH error occured");
 
 	TRY_nomem(m_signal_str[1] = "[HUP]: Hangup");
 	TRY_nomem(m_signal_str[2] = "[INT]: Interrupt ");
@@ -419,9 +420,9 @@ void job_archiver::mf_do_chores(void)
 	for (pi = m_job->paths.begin(); pi != m_job->paths.end(); ++pi) {
 		estring archive_dir;
 		estring path;
-		// estring command_line;
 		std::string binary;
 		std::vector<std::string> argv;
+		std::vector<std::string> extra_argv;
 		estring opt;
 		bool hardlink = false;
 		bool multi_hardlink = false;
@@ -438,28 +439,11 @@ void job_archiver::mf_do_chores(void)
 		estring error_msg;
 
 		archive_dir = m_job->generate_archive_path(*pi);
-		// If archive_dir does not end with a '/', strip off characters until it
-		// does.
-		/*
-		while ((archive_dir.size() > 0) 
-			&& (archive_dir[archive_dir.size()-1] != '/'))
-		{
-			archive_dir.erase(archive_dir.size()-1);
-		}
-		*/
 
 		path = archiver.working_archive_path();
 		path += "/";
 		path += archive_dir;
 		path = reform_path(path);
-		/*
-		if (!exists(path)) {
-			std::cout << "Creating job archive path: " << archive_dir << std::endl;
-			mk_dirhier(path);
-		}
-		else
-			std::cout << "Archiving to existing path: " << archive_dir << std::endl;
-		*/
 
 		if (!exists(path)) {
 			std::string es;
@@ -493,35 +477,39 @@ void job_archiver::mf_do_chores(void)
 			exit_code = 0;
 			signal_num = 0;
 
-			/*
-			command_line = config.rsync_local_path();
-			command_line += " ";
-			command_line += m_job->rsync_options;
-			*/
-			/**/
-			binary = config.rsync_local_path();
-			argv = m_job->generate_rsync_options_vector();
-			/**/
+			if (m_job->rsync_connection == job::connection_ssh_local) {
+				binary = config.ssh_local_path();
+			}
+			else {
+				binary = config.rsync_local_path();
+			}
+
+			argv.clear();
+			if (m_job->rsync_connection == job::connection_ssh_local) {
+				argv.push_back("-o");
+				argv.push_back("BatchMode=yes");
+				extra_argv = m_job->generate_ssh_options_vector();
+				argv.insert(argv.end(), extra_argv.begin(), extra_argv.end());
+				argv.push_back("-l");
+				argv.push_back(m_job->rsync_remote_user);
+				argv.push_back(m_job->hostname);
+				if (m_job->rsync_remote_path.size() != 0)
+					argv.push_back(m_job->rsync_remote_path);
+				else
+					argv.push_back(config.rsync_local_path());
+			}
+
+			extra_argv = m_job->generate_rsync_options_vector();
+			argv.insert(argv.end(), extra_argv.begin(), extra_argv.end());
+
 
 			if (m_job->rsync_connection != job::connection_local) {
-				/*
-				command_line += " ";
-				command_line += " --rsync-path=";
-				if (m_job->rsync_remote_path.size()) {
-					command_line += m_job->rsync_remote_path;
-				}
-				else {
-					command_line += config.rsync_local_path();
-				}
-				*/
-				/**/
 				opt = "--rsync-path=";
 				if (m_job->rsync_remote_path.size() != 0)
 					opt += m_job->rsync_remote_path;
 				else
 					opt += config.rsync_local_path();
 				argv.push_back(opt);
-				/**/
 			}
 
 			if (hardlink) {
@@ -565,27 +553,17 @@ void job_archiver::mf_do_chores(void)
 								<< "Using archive as hardlink source: " 
 								<< *si
 								<< std::endl;
-							// youngest = ypath;
 							if (!hardlink_opt) {
-								/**/
 								argv.push_back(std::string("--hard-links"));
-								/**/
-								/*
-								command_line += " --hard-links";
-								*/
 								hardlink_opt = true;
 							}
 
 							relative_path = mk_relative_path(ypath,path);
-							/*
-							command_line += " --link-dest=";
-							command_line += relative_path;
-							*/
-							/**/
+
 							opt="--link-dest=";
 							opt += relative_path;
 							argv.push_back(opt);
-							/**/
+
 							first_hardlink = true;
 							linkdest_count++; // At most 20 link-dest options can be used w/ rsync
 						}
@@ -597,13 +575,6 @@ void job_archiver::mf_do_chores(void)
 						}
 					}
 				}
-				/*
-				if (youngest.size() > 0) {
-					relative_path = mk_relative_path(youngest,path);
-					command_line += " --hard-links --link-dest=";
-					command_line += relative_path;
-				}
-				*/
 			}
 
 			for (
@@ -612,15 +583,9 @@ void job_archiver::mf_do_chores(void)
 				++ei
 				)
 			{
-				/*
-				command_line += " --exclude-from=";
-				command_line += *ei;
-				*/
-				/**/
 				opt = "--exclude-from=";
 				opt += *ei;
 				argv.push_back(opt);
-				/**/
 			}
 
 			for (
@@ -629,37 +594,15 @@ void job_archiver::mf_do_chores(void)
 				++ii
 				)
 			{
-				/*
-				command_line += " --include-from=";
-				command_line += *ii;
-				*/
-				/**/
 				opt = "--include-from=";
 				opt += *ei;
 				argv.push_back(opt);
-				/**/
 			}
 
-			/*
-			command_line += " ";
-			command_line += m_job->generate_source_path(*pi);
-			*/
-			/**/
 			argv.push_back(m_job->generate_source_path(*pi));
-			/**/
 			
-			/*
-			command_line += " ";
-			command_line += path;
-			*/
-			/**/
 			argv.push_back(path);
-			/**/
 	
-			/*
-			std::cout << "Command Line: " << command_line << std::endl;
-			*/
-			/**/
 			std::cout << "Command Line:" << std::endl;
 			{
 				uint16 c;
@@ -668,19 +611,15 @@ void job_archiver::mf_do_chores(void)
 					std::cout << "    Argv[" << c << "] = " << argv[c] << std::endl;
 				}
 			}
-			/**/
 
 			m_error_msg.erase();
 
 			t.start();
-			/*
-			exec.exec(command_line);
-			*/
-			/**/
+
 			m_io_timer.start();
 			m_rsync_timeout_flag = false;
 			exec.exec(binary, argv);
-			/**/
+
 			mf_process_rsync_io(
 				exec, 
 				m_job->rsync_timeout,
@@ -1318,6 +1257,17 @@ void archive_manager::archive(void)
 			lstr += archive_path();
 			lstr += "\n";
 			logger.write(lstr);
+
+			if (!writable(archive_path())) {
+				std::string es;
+
+				TRY_nomem(es = "Cannot write to archive directory: \"");
+				TRY_nomem(es += archive_path());
+				TRY_nomem(es += "\"");
+
+				throw(ERROR(EACCES,es));
+			}
+
 			rename_file(archive_path(), working_archive_path());
 		}
 		else if (exists(working_archive_path())) {
@@ -1325,6 +1275,16 @@ void archive_manager::archive(void)
 			lstr += working_archive_path();
 			lstr += "\n";
 			logger.write(lstr);
+
+			if (!writable(working_archive_path())) {
+				std::string es;
+
+				TRY_nomem(es = "Cannot write to working archive directory: \"");
+				TRY_nomem(es += working_archive_path());
+				TRY_nomem(es += "\"");
+
+				throw(ERROR(EACCES,es));
+			}
 		}
 		else {
 			lstr = "Creating archive directory: ";
