@@ -102,6 +102,7 @@ void estring::T_fraction_to_strings(const T& a_t,
 	value_type& a_ws, value_type& a_fs)
 {
 	T fraction = a_t;
+	uint64 rounded;
 	bool negative = false;
 	uint64 whole_part;
 	uint64 fractional_part;
@@ -121,7 +122,23 @@ void estring::T_fraction_to_strings(const T& a_t,
 		fraction = -fraction;
 	}
 
-	whole_part = static_cast<uint64>(fraction);
+	multiplier = m_base;
+	for (c = 0; c < m_precision; c++) {
+		multiplier *= m_base;
+	}
+
+	rounded = static_cast<uint64>(fraction * multiplier);
+	fractional_part = rounded % multiplier;
+	whole_part = rounded / multiplier;
+
+	if ((fractional_part % m_base) + (m_base/2) >= m_base) {
+		fractional_part += m_base/2;
+		if (fractional_part > (fractional_part % multiplier)) {
+			whole_part++;
+		}
+		fractional_part %= multiplier;
+	}
+
 	if (whole_part == 0) {
 		TRY_nomem(a_ws = "0");
 	}
@@ -134,20 +151,17 @@ void estring::T_fraction_to_strings(const T& a_t,
 		}
 	}
 
-	multiplier = m_base;
-	for (c = 0; c < m_precision; c++)
-		multiplier *= m_base;
-
-	fractional_part = static_cast<uint64>(
-		(fraction - static_cast<uint64>(fraction)) * multiplier);
-	fractional_part += (m_base/2);
 	fractional_part /= m_base;
-
-	while (fractional_part > 0) {
-		digit = fractional_part % m_base;
-		ch = m_alphabet[digit];
-		TRY_nomem(a_fs = ch +a_fs);
-		fractional_part /= m_base;
+	if (fractional_part == 0) {
+		TRY_nomem(a_fs = "0");
+	}
+	else {
+		while (fractional_part > 0) {
+			digit = fractional_part % m_base;
+			ch = m_alphabet[digit];
+			TRY_nomem(a_fs = ch +a_fs);
+			fractional_part /= m_base;
+		}
 	}
 	if (a_fs.size() > m_precision) {
 		a_fs.erase(m_precision);
