@@ -612,7 +612,7 @@ void mk_dir(const std::string& a_path)
 void rm_dir(const std::string a_path)
 {
 	if (!exists(a_path)) return;
-	if (rmdir(a_path.c_str()) != 0) {
+	if ((rmdir(a_path.c_str()) != 0) && exists(a_path)) {
 		std::string es;
 
 		TRY_nomem(es = "Could not remove directory: \"");
@@ -637,7 +637,7 @@ void rm_dir(const std::string a_path)
 void rm_file(const std::string a_path)
 {
 	if (!exists(a_path)) return;
-	if (unlink(a_path.c_str()) != 0) {
+	if ((unlink(a_path.c_str()) != 0) && exists(a_path)) {
 		std::string es;
 
 		TRY_nomem(es = "Could not remove file: \"");
@@ -734,6 +734,12 @@ void rename_file(const std::string a_from, const std::string a_to)
 		TRY_nomem(es += "\"");
 		throw(ERROR(0,es));
 	}
+	if (!writable(a_from)) {
+		TRY_nomem(es = "From filename is not writable: \"");
+		TRY_nomem(es += a_from);
+		TRY_nomem(es += "\"");
+		throw(ERROR(0,es));
+	}
 	if (rename(a_from.c_str(), a_to.c_str()) != 0) {
 		TRY_nomem(es = "Could not rename file: \"");
 		TRY_nomem(es += a_from);
@@ -1412,6 +1418,12 @@ void rm_recursive(const std::string a_path)
 	filestatus f;
 	std::string pathstr;
 	std::string es;
+	mode_t mode = 0;
+	
+	mode = 0;
+	mode |= S_IWUSR|S_IXUSR;
+	mode |= S_IWGRP|S_IXGRP;
+	mode |= S_IWOTH|S_IXOTH;
 
 	TRY_nomem(es = "Could not recursively delete: \"");
 	TRY_nomem(es += a_path);
@@ -1419,6 +1431,10 @@ void rm_recursive(const std::string a_path)
 
 	if (!exists(a_path)) {
 		if (is_link(a_path)) {
+			if (!writable(a_path)) {
+				if (chmod(a_path.c_str(), mode) != 0)
+					throw(ERROR(errno,es));
+			}
 			TRY(rm_file(a_path),es);
 		}
 		return;
@@ -1436,16 +1452,32 @@ void rm_recursive(const std::string a_path)
 						);
 				TRY(f.path(pathstr),es);
 				if (f.is_directory()) {
+					if (!writable(a_path)) {
+						if (chmod(a_path.c_str(), mode) != 0)
+							throw(ERROR(errno,es));
+					}
 					TRY(rm_recursive(pathstr),es);
 				}
 				else {
+					if (!writable(a_path)) {
+						if (chmod(a_path.c_str(), mode) != 0)
+							throw(ERROR(errno,es));
+					}
 					rm_file(pathstr);
 				}
 			}
 		}
+		if (!writable(a_path)) {
+			if (chmod(a_path.c_str(), mode) != 0)
+				throw(ERROR(errno,es));
+		}
 		rm_dir(a_path);
 	}
 	else {
+		if (!writable(a_path)) {
+			if (chmod(a_path.c_str(), mode) != 0)
+				throw(ERROR(errno,es));
+		}
 		rm_file(a_path);
 	}
 }
