@@ -586,6 +586,7 @@ void job::clear(void)
 	rsync_multi_hardlink = false;
 	rsync_multi_hardlink_max = 20;
 	rsync_options.erase();
+	ssh_options.erase();
 	rsync_remote_user.erase();
 	rsync_remote_path.erase();
 	rsync_remote_port = 0;
@@ -618,6 +619,7 @@ void job::assign(const job& a_job)
 	rsync_multi_hardlink = a_job.rsync_multi_hardlink;
 	rsync_multi_hardlink_max = a_job.rsync_multi_hardlink_max;
 	rsync_options = a_job.rsync_options;
+	ssh_options = a_job.ssh_options;
 	rsync_remote_user = a_job.rsync_remote_user;
 	rsync_remote_path = a_job.rsync_remote_path;
 	rsync_remote_port = a_job.rsync_remote_port;
@@ -866,6 +868,7 @@ const std::string job::generate_job_id(void) const
 	return(path);
 }
 
+/** Generate rsync command line options */
 const std::vector<std::string> job::generate_rsync_options_vector(void) const
 {
 	std::string opts = rsync_options;
@@ -937,6 +940,78 @@ const std::vector<std::string> job::generate_rsync_options_vector(void) const
 	return(argv);
 }
 
+/** Generate ssh command line options */
+const std::vector<std::string> job::generate_ssh_options_vector(void) const
+{
+	std::string opts = ssh_options;
+	std::vector<std::string> argv;
+	std::string str;
+
+	while (opts.size() != 0) {
+
+		str.clear();
+		while ((opts.size() != 0) && (opts[0] != ' ') && (opts[0] != '\t')) {
+			switch (opts[0]) {
+				case '\0':
+					break;
+				case '\'':
+					opts.erase(0,1);
+					while ((opts.size() != 0) && (opts[0] != '\'')) {
+						str += opts[0];
+						opts.erase(0,1);
+					}
+					if (opts[0] == '\'') {
+						opts.erase(0,1);
+					}
+					break;
+				case '"':
+					opts.erase(0,1);
+					while ((opts.size() != 0) && (opts[0] != '"')) {
+						if ((opts.size() >= 2) && (opts[0] == '\\') && (opts[1] == '"')) {
+							str += '"';
+							opts.erase(0,2);
+						}
+						else {
+							str += opts[0];
+							opts.erase(0,1);
+						}
+					}
+					if (opts[0] == '"') {
+						opts.erase(0,1);
+					}
+					break;
+				case '\\':
+					if ((opts.size() >= 2) && (opts[1] == ' ')) {
+						str += ' ';
+						opts.erase(0,2);
+					}
+					else if ((opts.size() >= 2) && (opts[1] == 't')) {
+						str += '\t';
+						opts.erase(0,2);
+					}
+					else {
+						str += '\\';
+						opts.erase(0,1);
+					}
+					break;
+				default:
+					str += opts[0];
+					opts.erase(0,1);
+					break;
+			}
+		}
+
+		if (str.size()) {
+			argv.push_back(str);
+		}
+		while ((opts.size() > 0) && ((opts[0] == ' ') || (opts[0] == '\t'))) {
+			opts.erase(0,1);
+		}
+	}
+
+	return(argv);
+}
+
 /** Perform sanity checks for the configuration settings of this job */
 void job::check(void)
 {
@@ -951,6 +1026,7 @@ void job::check(void)
 			(
 				(rsync_connection == connection_remote) 
 				|| (rsync_connection == connection_server)
+				|| (rsync_connection == connection_ssh_local)
 			)
 			&& (hostname.size() == 0)
 		)
@@ -960,6 +1036,28 @@ void job::check(void)
 		throw(ERROR(0,es));
 	}
 
+	if (rsync_connection == connection_ssh_local) {
+		/*
+		if (config.ssh_local_path.size() == 0) {
+			TRY_nomem(es = "rsync-connection-type references remote-rsync-local-dir, ");
+			TRY_nomem(es += "but ssh-local-path is empty");
+			throw(ERROR(0,es));
+		}
+		*/
+
+		if (excludes.size() != 0) {
+			TRY_nomem(es = "rsync-connection-type is remote-rsync-local-dir, ");
+			TRY_nomem(es += "but exclude files have been specified");
+			throw(ERROR(0,es));
+		}
+
+		if (includes.size() != 0) {
+			TRY_nomem(es = "rsync-connection-type is remote-rsync-local-dir, ");
+			TRY_nomem(es += "but include files have been specified");
+			throw(ERROR(0,es));
+		}
+	}
+
 	if ((rsync_remote_module.size() != 0) 
 		&& (rsync_connection != connection_server)) 
 	{
@@ -1084,9 +1182,14 @@ void configuration_manager::clear(void)
 	m_delete_old_log_files = false;
 	m_delete_old_report_files = false;
 	m_rsync_local_path.erase();
+	m_delete_command_path.erase();
 	if (strlen(LOCAL_RSYNC) > 0) {
 		TRY_nomem(m_rsync_local_path = LOCAL_RSYNC);
 	}
+	m_ssh_local_path.erase();
+	if (strlen(LOCAL_SSH) > 0) {
+		TRY_nomem(m_ssh_local_path = LOCAL_SSH);
+	}
 	m_rsync_parallel = 1;
 	m_io_poll_interval = 1;
 	m_vaults.clear();
@@ -1300,6 +1403,28 @@ void configuration_manager::check(void) const
 	TRY_nomem(es += "\"");
 	TRY(fstat.path(m_rsync_local_path),es);
 
+	// delete-command-path -- If delete-command-path is set, check to see if the
+	// command exists
+	if (m_delete_command_path.size() > 0) {
+		TRY_nomem(es = "Invalid delete-command-path value: \"");
+		TRY_nomem(es += m_delete_command_path);
+		TRY_nomem(es += "\"");
+		TRY(fstat.path(m_delete_command_path),es);
+	}
+
+	// ssh-local-path -- If (a) ssh-local-path is not set from config, and
+	// (b) a default was determined at compile time, then (c) does the default
+	// exist?
+	//
+	// TODO: NOT SURE WE SHOULD CARE ABOUT THIS HERE.  What if the end user
+	// doesn't want or need to have ssh installed on their machine?
+	/*
+	TRY_nomem(es = "Invalid ssh-local-path value: \"");
+	TRY_nomem(es += m_ssh_local_path);
+	TRY_nomem(es += "\"");
+	TRY(fstat.path(m_ssh_local_path),es);
+	*/
+
 	// vault -- Are there any?
 	if (m_vaults.size() == 0) {
 		throw(ERROR(0,"No vaults defined"));
@@ -1430,6 +1555,24 @@ const std::string& configuration_manager::rsync_local_path(void) const
 	return(m_rsync_local_path);
 }
 
+/** Return the delete-command-path */
+const std::string& configuration_manager::delete_command_path(void) const
+{
+	if (!initialized())
+		throw(INTERNAL_ERROR(0,"Configuration manager is not initialized"));
+
+	return(m_delete_command_path);
+}
+
+/** Return the ssh-local-path */
+const std::string& configuration_manager::ssh_local_path(void) const
+{
+	if (!initialized())
+		throw(INTERNAL_ERROR(0,"Configuration manager is not initialized"));
+
+	return(m_ssh_local_path);
+}
+
 /** Return the rsync-parallel */
 const uint16& configuration_manager::rsync_parallel(void) const
 {
@@ -1837,6 +1980,12 @@ void global_parser::parse(void)
 			else if (keyword == "rsync-local-path") {
 				parse_rsync_local_path(value);
 			}
+			else if (keyword == "delete-command-path") {
+				parse_delete_command_path(value);
+			}
+			else if (keyword == "ssh-local-path") {
+				parse_ssh_local_path(value);
+			}
 			else if (keyword == "rsync-parallel") {
 				parse_rsync_parallel(value);
 			}
@@ -2205,6 +2354,32 @@ void global_parser::parse_rsync_local_path(const std::string& a_value)
 	TRY_nomem(config.m_rsync_local_path = a_value);
 }
 
+/** Parse an "delete-command-path" command */
+void global_parser::parse_delete_command_path(const std::string& a_value)
+{
+	std::string es;
+	filestatus fstat;
+
+	TRY_nomem(es = "Invalid delete-command-path value: \"");
+	TRY_nomem(es += a_value);
+	TRY_nomem(es += "\"");
+	TRY_instead(fstat.path(a_value),es);
+	TRY_nomem(config.m_delete_command_path = a_value);
+}
+
+/** Parse an "ssh-local-path" command */
+void global_parser::parse_ssh_local_path(const std::string& a_value)
+{
+	std::string es;
+	filestatus fstat;
+
+	TRY_nomem(es = "Invalid ssh-local-path value: \"");
+	TRY_nomem(es += a_value);
+	TRY_nomem(es += "\"");
+	TRY_instead(fstat.path(a_value),es);
+	TRY_nomem(config.m_ssh_local_path = a_value);
+}
+
 /** Parse an "rsync-parallel" command */
 void global_parser::parse_rsync_parallel(const std::string& a_value)
 {
@@ -2585,9 +2760,15 @@ void job_parser::parse(void)
 			else if (keyword == "rsync-options") {
 				parse_rsync_options(value);
 			}
+			else if (keyword == "ssh-options") {
+				parse_ssh_options(value);
+			}
 			else if (keyword == "<rsync-options>") {
 				parse_rsync_options_context(value);
 			}
+			else if (keyword == "<ssh-options>") {
+				parse_ssh_options_context(value);
+			}
 			else if (keyword == "rsync-remote-user") {
 				parse_rsync_remote_user(value);
 			}
@@ -2685,6 +2866,9 @@ void job_parser::parse_clear(const std::string& a_value)
 	else if (str == "rsync-options") {
 		m_job->rsync_options.erase();
 	}
+	else if (str == "ssh-options") {
+		m_job->ssh_options.erase();
+	}
 	else if (str == "rsync-remote-user") {
 		m_job->rsync_remote_user.erase();
 	}
@@ -2886,6 +3070,9 @@ void job_parser::parse_rsync_connection_type(const std::string& a_value)
 	else if (str == "server") {
 		(*m_job).rsync_connection = job::connection_server;
 	}
+	else if (str == "remote-rsync-local-dir") {
+		(*m_job).rsync_connection = job::connection_ssh_local;
+	}
 	else {
 		throw(ERROR(0,es));
 	}
@@ -2991,6 +3178,20 @@ void job_parser::parse_rsync_options(const std::string& a_value)
 	}
 }
 
+/** Parse an "ssh-options" command */
+void job_parser::parse_ssh_options(const std::string& a_value)
+{
+	if (*m_delimiter == "</default>") {
+		TRY_nomem((*m_job).ssh_options = a_value);
+	}
+	else {
+		if ((*m_job).ssh_options.size() != 0) {
+			TRY_nomem((*m_job).ssh_options += " ");
+		}
+		TRY_nomem((*m_job).ssh_options += a_value);
+	}
+}
+
 /** Parse an rsync-options context */
 void job_parser::parse_rsync_options_context(const std::string& a_value)
 {
@@ -3035,6 +3236,50 @@ void job_parser::parse_rsync_options_context(const std::string& a_value)
 	}
 }
 
+/** Parse an ssh-options context */
+void job_parser::parse_ssh_options_context(const std::string& a_value)
+{
+	std::string es;
+	std::string value;
+	std::string ssh_options;
+	char ch;
+
+	while ((*m_in)) {
+		(*m_line)++;
+
+		value.erase();
+
+		(*m_in).get(ch);
+		while ((ch == ' ') || (ch == '\t'))
+			(*m_in).get(ch);
+		while (((*m_in)) && (ch != '\n')) {
+			TRY_nomem(value += ch);
+			(*m_in).get(ch);
+		}
+
+		if ((!(*m_in)) && (value != "</ssh-options>")) {
+			TRY_nomem(es = "Unexpected end of file, expected ");
+			TRY_nomem(es += "\"</ssh-options>\" here");
+			throw(ERROR(errno,es));
+		}
+
+		if (value.size() == 0)
+			continue;
+		if (value[0] == '#')
+			continue;
+
+		if (value == "</ssh-options>") {
+			parse_ssh_options(ssh_options);
+			return;
+		}
+
+		if (ssh_options.size() != 0) {
+			TRY_nomem(ssh_options += ' ');
+		}
+		TRY_nomem(ssh_options += value);
+	}
+}
+
 /** Parse a "remote-user" command */
 void job_parser::parse_rsync_remote_user(const std::string& a_value)
 {
