@@ -439,6 +439,7 @@ void vault_manager::delete_oldest_archive(void)
 	subdirectory archive_list; 
 	std::string oldest;
 	std::string dir;
+	std::string delete_dir;
 	estring estr;
 	timer t;
 	uint16 free_blocks, free_inodes;
@@ -499,7 +500,27 @@ void vault_manager::delete_oldest_archive(void)
 	catch(...) {
 		m_da_err = true;
 	}
-	rm_recursive(dir);
+	
+	delete_dir = dir;
+	if (delete_dir.find(".deleting") == std::string::npos)
+		delete_dir += ".deleting";
+	rename_file(dir, delete_dir);
+	if (config.delete_command_path().size() == 0)
+		rm_recursive(delete_dir);
+	else {
+		std::string es;
+		std::string cmdline;
+
+		TRY_nomem(es = "Delete command returned non-zero exit value: \"");
+		TRY_nomem(es += oldest);
+		TRY_nomem(es += "\"");
+		cmdline = config.delete_command_path();
+		cmdline += " \"";
+		cmdline += delete_dir;
+		cmdline += "\"";
+		if (system(cmdline.c_str()) != 0)
+			throw(ERROR(1,es));
+	}
 	t.stop();
 
 	TRY_nomem(lstr = "Deletion complete, duration: ");
